@@ -61,6 +61,10 @@ def configure(repl):
     # completion menu is shown.)
     repl.complete_while_typing = True
 
+    # Fuzzy and dictionary completion.
+    self.enable_fuzzy_completion = False
+    self.enable_dictionary_completion = False
+
     # Vi mode.
     repl.vi_mode = False
 
@@ -132,9 +136,7 @@ def configure(repl):
     """
     @repl.add_key_binding(Keys.ControlE, Keys.ControlE)
     def _(event):
-        b = event.current_buffer
-        if b.accept_action.is_returnable:
-            b.accept_action.validate_and_handle(event.cli, b)
+        event.current_buffer.validate_and_handle()
     """
 
 
