@@ -6,7 +6,10 @@ from prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
 
 from ptpython.utils import get_jedi_script_from_document
 
+import keyword
+import ast
 import re
+import six
 
 __all__ = (
     'PythonCompleter',
@@ -17,11 +20,14 @@ class PythonCompleter(Completer):
     """
     Completer for Python code.
     """
-    def __init__(self, get_globals, get_locals):
+    def __init__(self, get_globals, get_locals, get_enable_dictionary_completion):
         super(PythonCompleter, self).__init__()
 
         self.get_globals = get_globals
         self.get_locals = get_locals
+        self.get_enable_dictionary_completion = get_enable_dictionary_completion
+
+        self.dictionary_completer = DictionaryCompleter(get_globals, get_locals)
 
         self._path_completer_cache = None
         self._path_completer_grammar_cache = None
@@ -108,7 +114,16 @@ class PythonCompleter(Completer):
         """
         Get Python completions.
         """
-        # Do Path completions
+        # Do dictionary key completions.
+        if self.get_enable_dictionary_completion():
+            has_dict_completions = False
+            for c in self.dictionary_completer.get_completions(document, complete_event):
+                has_dict_completions = True
+                yield c
+            if has_dict_completions:
+                return
+
+        # Do Path completions (if there were no dictionary completions).
         if complete_event.completion_requested or self._complete_path_while_typing(document):
             for c in self._path_completer.get_completions(document, complete_event):
                 yield c
@@ -154,7 +169,115 @@ class PythonCompleter(Completer):
                     # In jedi.api.helpers.py: 144, in get_stack_at_position
                     # raise SystemError("This really shouldn't happen. There's a bug in Jedi.")
                     pass
+                except NotImplementedError:
+                    # See: https://github.com/jonathanslenders/ptpython/issues/223
+                    pass
+                except Exception:
+                    # Supress all other Jedi exceptions.
+                    pass
                 else:
                     for c in completions:
-                        yield Completion(c.name_with_symbols, len(c.complete) - len(c.name_with_symbols),
-                                         display=c.name_with_symbols)
+                        yield Completion(
+                            c.name_with_symbols, len(c.complete) - len(c.name_with_symbols),
+                            display=c.name_with_symbols,
+                            style=_get_style_for_name(c.name_with_symbols))
+
+
+class DictionaryCompleter(Completer):
+    """
+    Experimental completer for Python dictionary keys.
+
+    Warning: This does an `eval` on the Python object before the open square
+             bracket, which is potentially dangerous. It doesn't match on
+             function calls, so it only triggers attribute access.
+    """
+    def __init__(self, get_globals, get_locals):
+        super(DictionaryCompleter, self).__init__()
+
+        self.get_globals = get_globals
+        self.get_locals = get_locals
+
+        self.pattern = re.compile(
+            r'''
+                # Any expression safe enough to eval while typing.
+                # No operators, except dot, and only other dict lookups.
+                # Technically, this can be unsafe of course, if bad code runs
+                # in `__getattr__` or ``__getitem__``.
+                (
+                    # Variable name
+                    [a-zA-Z0-9_]+
+
+                    \s*
+
+                    (?:
+                        # Attribute access.
+                        \s* \. \s* [a-zA-Z0-9_]+ \s*
+
+                        |
+
+                        # Item lookup.
+                        # (We match the square brackets. We don't care about
+                        # matching quotes here in the regex. Nested square
+                        # brackets are not supported.)
+                        \s* \[ [a-zA-Z0-9_'"\s]+ \] \s*
+                    )*
+                )
+
+                # Dict loopup to complete (square bracket open + start of
+                # string).
+                \[
+                \s* ([a-zA-Z0-9_'"]*)$
+            ''',
+            re.VERBOSE
+        )
+
+    def get_completions(self, document, complete_event):
+        match = self.pattern.search(document.text_before_cursor)
+        if match is not None:
+            object_var, key = match.groups()
+            object_var = object_var.strip()
+
+            # Do lookup of `object_var` in the context.
+            try:
+                result = eval(object_var, self.get_globals(), self.get_locals())
+            except BaseException as e:
+                return  # Many exception, like NameError can be thrown here.
+
+            # If this object is a dictionary, complete the keys.
+            if isinstance(result, dict):
+                # Try to evaluate the key.
+                key_obj = key
+                for k in [key, key + '"', key + "'"]:
+                    try:
+                        key_obj = ast.literal_eval(k)
+                    except (SyntaxError, ValueError):
+                        continue
+                    else:
+                        break
+
+                for k in result:
+                    if six.text_type(k).startswith(key_obj):
+                        yield Completion(
+                            six.text_type(repr(k)),
+                            - len(key),
+                            display=six.text_type(repr(k))
+                        )
+
+try:
+    import builtins
+    _builtin_names = dir(builtins)
+except ImportError:  # Python 2.
+    _builtin_names = []
+
+
+def _get_style_for_name(name):
+    """
+    Return completion style to use for this name.
+    """
+    if name in _builtin_names:
+        return 'class:completion.builtin'
+
+    if keyword.iskeyword(name):
+        return 'class:completion.keyword'
+
+    return ''
