@@ -20,14 +20,14 @@ import six
 import sys
 
 
-def run():
+def run(user_ns=None):
     a = docopt.docopt(__doc__)
 
     vi_mode = bool(a['--vi'])
-    config_dir = os.path.expanduser(a['--config-dir'] or os.path.join('~', '.ptpython')
+    config_dir = os.path.expanduser(a['--config-dir'] or '~/.ptpython/')
 
     # Create config directory.
-    if not os.path.isdir(config_dir) or not os.path.islink(config_dir):
+    if not os.path.isdir(config_dir):
         os.mkdir(config_dir)
 
     # If IPython is not available, show message and exit here with error status
@@ -48,14 +48,18 @@ def run():
     # When a file has been given, run that, otherwise start the shell.
     if a['<arg>'] and not a['--interactive']:
         sys.argv = a['<arg>']
-        six.exec_(compile(open(a['<arg>'][0], "rb").read(), a['<arg>'][0], 'exec'))
+        path = a['<arg>'][0]
+        with open(path, 'rb') as f:
+            code = compile(f.read(), path, 'exec')
+            six.exec_(code)
     else:
         enable_deprecation_warnings()
 
         # Create an empty namespace for this interactive shell. (If we don't do
         # that, all the variables from this function will become available in
         # the IPython shell.)
-        user_ns = {}
+        if user_ns is None:
+            user_ns = {}
 
         # Startup path
         startup_paths = []
@@ -70,7 +74,7 @@ def run():
         # exec scripts from startup paths
         for path in startup_paths:
             if os.path.exists(path):
-                with open(path, 'r') as f:
+                with open(path, 'rb') as f:
                     code = compile(f.read(), path, 'exec')
                     six.exec_(code, user_ns, user_ns)
             else:
