@@ -11,16 +11,17 @@ offer.
 from __future__ import unicode_literals, print_function
 
 from prompt_toolkit.completion import Completion, Completer
-from prompt_toolkit.contrib.completers import PathCompleter, WordCompleter, SystemCompleter
+from prompt_toolkit.completion import PathCompleter, WordCompleter
+from prompt_toolkit.contrib.completers import SystemCompleter
 from prompt_toolkit.contrib.regular_languages.compiler import compile
 from prompt_toolkit.contrib.regular_languages.completion import GrammarCompleter
 from prompt_toolkit.contrib.regular_languages.lexer import GrammarLexer
 from prompt_toolkit.document import Document
-from prompt_toolkit.interface import CommandLineInterface
-from prompt_toolkit.layout.lexers import PygmentsLexer, SimpleLexer
+from prompt_toolkit.formatted_text import PygmentsTokens
+from prompt_toolkit.lexers import PygmentsLexer, SimpleLexer
+from prompt_toolkit.styles import Style
 
 from .python_input import PythonInput, PythonValidator, PythonCompleter
-from .eventloop import create_eventloop
 from .style import default_ui_style
 
 from IPython.terminal.embed import InteractiveShellEmbed as _InteractiveShellEmbed
@@ -29,7 +30,6 @@ from IPython import utils as ipy_utils
 from IPython.core.inputsplitter import IPythonInputSplitter
 
 from pygments.lexers import PythonLexer, BashLexer
-from pygments.token import Token
 from ptpython.prompt_style import PromptStyle
 
 __all__ = (
@@ -38,42 +38,19 @@ __all__ = (
 
 
 class IPythonPrompt(PromptStyle):
-    """
-    PromptStyle that uses the templates, as set by IPython.
-    Usually, something like "In [1]:".
-    """
-    def __init__(self, prompt_manager):
-        self.prompt_manager = prompt_manager
-
-    def in_tokens(self, cli):
-        text = self.prompt_manager.render('in', color=False, just=False)
-        return [(Token.In, text)]
-
-    def in2_tokens(self, cli, width):
-        text = self.prompt_manager.render('in2', color=False, just=False)
-        return [(Token.In, text.rjust(width))]
-
-    def out_tokens(self, cli):
-        # This function is currently not used by IPython. But for completeness,
-        # it would look like this.
-        text = self.prompt_manager.render('out', color=False, just=False)
-        return [(Token.Out, text)]
-
-
-class IPython5Prompt(PromptStyle):
     """
     Style for IPython >5.0, use the prompt_toolkit tokens directly.
     """
     def __init__(self, prompts):
         self.prompts = prompts
 
-    def in_tokens(self, cli):
-        return self.prompts.in_prompt_tokens(cli)
+    def in_prompt(self):
+        return PygmentsTokens(self.prompts.in_prompt_tokens())
 
-    def in2_tokens(self, cli, width):
-        return self.prompts.continuation_prompt_tokens(cli)
+    def in2_prompt(self, width):
+        return PygmentsTokens(self.prompts.continuation_prompt_tokens())
 
-    def out_tokens(self, cli):
+    def out_prompt(self):
         return []
 
 
@@ -113,11 +90,12 @@ def create_ipython_grammar():
     """)
 
 
-def create_completer(get_globals, get_locals, magics_manager, alias_manager):
+def create_completer(get_globals, get_locals, magics_manager, alias_manager,
+                     get_enable_dictionary_completion):
     g = create_ipython_grammar()
 
     return GrammarCompleter(g, {
-        'python': PythonCompleter(get_globals, get_locals),
+        'python': PythonCompleter(get_globals, get_locals, get_enable_dictionary_completion),
         'magic': MagicsCompleter(magics_manager),
         'alias_name': AliasCompleter(alias_manager),
         'pdb_arg': WordCompleter(['on', 'off'], ignore_case=True),
@@ -135,9 +113,9 @@ def create_lexer():
     return GrammarLexer(
         g,
         lexers={
-            'percent': SimpleLexer(Token.Operator),
-            'magic': SimpleLexer(Token.Keyword),
-            'filename': SimpleLexer(Token.Name),
+            'percent': SimpleLexer('class:pygments.operator'),
+            'magic': SimpleLexer('class:pygments.keyword'),
+            'filename': SimpleLexer('class:pygments.name'),
             'python': PygmentsLexer(PythonLexer),
             'system': PygmentsLexer(BashLexer),
         })
@@ -177,7 +155,8 @@ class IPythonInput(PythonInput):
     def __init__(self, ipython_shell, *a, **kw):
         kw['_completer'] = create_completer(kw['get_globals'], kw['get_globals'],
                                             ipython_shell.magics_manager,
-                                            ipython_shell.alias_manager)
+                                            ipython_shell.alias_manager,
+                                            lambda: self.enable_dictionary_completion)
         kw['_lexer'] = create_lexer()
         kw['_validator'] = IPythonValidator(
             get_compiler_flags=self.get_compiler_flags)
@@ -185,29 +164,21 @@ class IPythonInput(PythonInput):
         super(IPythonInput, self).__init__(*a, **kw)
         self.ipython_shell = ipython_shell
 
-        # Prompt for IPython < 5.0
-        if hasattr(ipython_shell, 'prompt_manager'):
-            self.all_prompt_styles['ipython'] = IPythonPrompt(ipython_shell.prompt_manager)
-            self.prompt_style = 'ipython'
-
-        # Prompt for IPython >=5.0:
-        if hasattr(ipython_shell, 'prompts'):
-            self.all_prompt_styles['ipython'] = IPython5Prompt(ipython_shell.prompts)
-            self.prompt_style = 'ipython'
-
+        self.all_prompt_styles['ipython'] = IPythonPrompt(ipython_shell.prompts)
+        self.prompt_style = 'ipython'
 
         # UI style for IPython. Add tokens that are used by IPython>5.0
         style_dict = {}
         style_dict.update(default_ui_style)
         style_dict.update({
-            Token.Prompt:        '#009900',
-            Token.PromptNum:     '#00ff00 bold',
-            Token.OutPrompt:     '#990000',
-            Token.OutPromptNum:  '#ff0000 bold',
+            'pygments.prompt':        '#009900',
+            'pygments.prompt-num':     '#00ff00 bold',
+            'pygments.out-prompt':     '#990000',
+            'pygments.out-prompt-num':  '#ff0000 bold',
         })
 
         self.ui_styles = {
-            'default': style_dict,
+            'default': Style.from_dict(style_dict),
         }
         self.use_ui_colorscheme('default')
 
@@ -225,45 +196,33 @@ class InteractiveShellEmbed(_InteractiveShellEmbed):
         configure = kw.pop('configure', None)
         title = kw.pop('title', None)
 
+        # Don't ask IPython to confirm for exit. We have our own exit prompt.
+        self.confirm_exit = False
+
         super(InteractiveShellEmbed, self).__init__(*a, **kw)
 
         def get_globals():
             return self.user_ns
 
-        self._eventloop = create_eventloop()
-        ipython_input = IPythonInput(
+        python_input = IPythonInput(
             self,
             get_globals=get_globals, vi_mode=vi_mode,
             history_filename=history_filename)
 
         if title:
-            ipython_input.terminal_title = title
+            python_input.terminal_title = title
 
         if configure:
-            configure(ipython_input)
-            ipython_input.prompt_style = 'ipython'  # Don't take from config.
+            configure(python_input)
+            python_input.prompt_style = 'ipython'  # Don't take from config.
 
-        self._cli = CommandLineInterface(
-                application=ipython_input.create_application(),
-                eventloop=self._eventloop)
+        self.python_input = python_input
 
     def prompt_for_code(self):
-        # IPython 5.0 calls `prompt_for_code` instead of `raw_input`.
-        return self.raw_input(self)
-
-    def raw_input(self, prompt=''):
-        print('')
         try:
-            string = self._cli.run(reset_current_buffer=True).text
-
-            # In case of multiline input, make sure to append a newline to the input,
-            # otherwise, IPython will ask again for more input in some cases.
-            if '\n' in string:
-                return string + '\n\n'
-            else:
-                return string
-        except EOFError:
-            self.ask_exit()
+            return self.python_input.app.run()
+        except KeyboardInterrupt:
+            self.python_input.default_buffer.document = Document()
             return ''
 
 
