@@ -20,6 +20,7 @@ from prompt_toolkit.utils import DummyContext
 from prompt_toolkit.shortcuts import set_title, clear_title
 from prompt_toolkit.shortcuts import print_formatted_text
 from prompt_toolkit.formatted_text import PygmentsTokens
+from prompt_toolkit.patch_stdout import patch_stdout as patch_stdout_context
 
 from .python_input import PythonInput
 from .eventloop import inputhook
@@ -75,8 +76,7 @@ class PythonRepl(PythonInput):
         if self.terminal_title:
             clear_title()
 
-    def _process_text(self, text):
-        line = self.default_buffer.text
+    def _process_text(self, line):
 
         if line and not line.isspace():
             try:
@@ -331,13 +331,19 @@ def embed(globals=None, locals=None, configure=None,
     app = repl.app
 
     # Start repl.
-    patch_context = app.patch_stdout_context() if patch_stdout else DummyContext()
+    patch_context = patch_stdout_context() if patch_stdout else DummyContext()
 
     if return_asyncio_coroutine: # XXX
         def coroutine():
             with patch_context:
-                for future in app.run_async():
-                    yield future
+                while True:
+                    iterator = iter(app.run_async().to_asyncio_future())
+                    try:
+                        while True:
+                            yield next(iterator)
+                    except StopIteration as exc:
+                        text = exc.args[0]
+                    repl._process_text(text)
         return coroutine()
     else:
         with patch_context:
